unit mainexifimage;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    BuCreateJPEG: TButton;
    BuCreateEXIF: TButton;
    BuReadEXIF: TButton;
    MemEXIF: TMemo;
    procedure BuCreateEXIFClick(Sender: TObject);
    procedure BuCreateJPEGClick(Sender: TObject);
    procedure BuReadEXIFClick(Sender: TObject);
  private

  public

  end;

var
  Form1: TForm1;

implementation
uses
  FPImage, FPCanvas, FPImgCanv, FPWriteJPEG, fpeMetadata, fpeTags;



const
 FNTest = 'DrawTest.jpg';

{$R *.lfm}

{ TForm1 }

procedure TForm1.BuCreateJPEGClick(Sender: TObject);
var canv : TFPcustomCanvas;
    image : TFPCustomImage;
    writer : TFPCustomImageWriter;
    {
      Colors range from 0 to 65535 in each primary color.
      They can also show as hexideciaml:
      $FFFF = 65535, $0000 = 0
    }
    passionRed: TFPColor = (Red: 65535; Green: 0; Blue: 0; Alpha: 65535);
begin
  image := TFPMemoryImage.Create (2000,5000);
  Canv := TFPImageCanvas.Create (image);
  Writer := TFPWriterJPEG.Create;

  { Set the pen styles }
  with canvas do
  begin
    pen.mode    := pmCopy;
    pen.style   := psSolid;
    pen.width   := 1;
    pen.FPColor := passionRed;
  end;

  { Draw a circle }
  canv.Ellipse (10,10, 90,90);

  { Save to file }
  image.SaveToFile (FNTest, writer);

  { Clean up! }
  Canv.Free;
  image.Free;
  writer.Free;
end;

procedure TForm1.BuReadEXIFClick(Sender: TObject);
var
  FImgInfo: TImgInfo;
  lTag: TTag;
begin
  try
    if FileExists(FNTest) then begin
      FImgInfo := TImgInfo.Create;
      try
        FImgInfo.LoadFromFile(FNTest);
        if not FImgInfo.HasExif then begin
          MemEXIF.Append('No EXIF found in '+FNTest);
          exit;
        end;
        lTag := FImgInfo.ExifData.TagByName['EXIF.SerialNumber'];
        if lTag = nil then begin
          MemEXIF.Append('EXIF Tag not found.');
        end
        else
          if lTag.ReadOnly then begin
            MemEXIF.Append('EXIF This tag is readonly.');
          end
          else
          if (lTag is TStringTag) then begin
            MemEXIF.Append('EXIF tag='+TStringTag(lTag).AsString);
          end;

      finally
        FImgInfo.free;
      end;
    end;
  except
    on E: Exception do begin
      MemEXIF.Append('EXIF Exception='+E.Message);
    end;
  end;
end;

procedure TForm1.BuCreateEXIFClick(Sender: TObject);
var
  FImgInfo: TImgInfo;
  lTag: TTag;

begin
  try
    if FileExists(FNTest) then begin
      FImgInfo := TImgInfo.Create;
      try
        FImgInfo.LoadFromFile(FNTest);
        // if not exif, create new
        if not FImgInfo.HasExif then begin
          FImgInfo.CreateExifData;
          MemEXIF.Append('No EXIF found in '+FNTest+' creating');
          FImgInfo.ExifData.AddTagByName('EXIF.SerialNumber').AsString:= '0987654321';
        end
        else begin
          MemEXIF.Append('EXIF.SerialNumber change');
          lTag := FImgInfo.ExifData.TagByName['EXIF.SerialNumber'];
          if lTag = nil then begin
            MemEXIF.Append('EXIF Tag not found.');
          end
          else
            if lTag.ReadOnly then begin
              MemEXIF.Append('EXIF This tag is readonly.');
            end
            else
            if (lTag is TStringTag) then begin
              TStringTag(lTag).AsString := '1234567890';
              MemEXIF.Append('EXIF This tag written.');
            end;
        end;
        FImgInfo.Save;
      finally
        FImgInfo.free;
      end;
    end;
  except
    on E: Exception do begin
      MemEXIF.Append('EXIF Exception='+E.Message);
    end;
  end;
end;

end.

